pub enum ProjectionKind {
Deref,
Field {
field_idx: u32,
variant_idx: u32,
},
Index,
Subslice,
UnwrapUnsafeBinder,
}Variants
Deref
A dereference of a pointer, reference or Box<T> of the given type.
Field
B.F where B is the base expression and F is
the field. The field is identified by which variant
it appears in along with a field index. The variant
is used for enums.
Index
Some index like B[x], where B is the base
expression. We don’t preserve the index x because
we won’t need it.
Subslice
A subslice covering a range of values like B[x..y].
UnwrapUnsafeBinder
unwrap_binder!(expr)